@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * SPDX-FileCopyrightText: (C) 1999-2008 Novell, Inc. (www.novell.com)
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -18,6 +17,7 @@
 #include "common/e-ews-camel-common.h"
 #include "common/e-ews-item-change.h"
 #include "common/e-ews-request.h"
+#include "e-ews-common-utils.h"
 
 #include "camel-ews-utils.h"
 
@@ -1549,6 +1549,10 @@ ews_utils_save_category_changes (GHashTable *old_categories, /* gchar *guid ~> C
 	gpointer value;
 	gboolean changed = FALSE;
 
+	/* cannot save, when evolution is not installed */
+	if (!e_ews_common_utils_gsettings_schema_exists ("org.gnome.evolution.mail"))
+		return FALSE;
+
 	if (!old_categories || !new_categories)
 		return new_categories != NULL;
 
