@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * SPDX-FileCopyrightText: (C) 2012 Red Hat, Inc. (www.redhat.com)
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -17,9 +16,6 @@ typedef struct _EEwsConfigUIExtensionClass EEwsConfigUIExtensionClass;
 struct _EEwsConfigUIExtension
 {
 	EExtension parent;
-
-	guint current_ui_id;
-	GHashTable *ui_definitions;
 };
 
 struct _EEwsConfigUIExtensionClass
