@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * SPDX-FileCopyrightText: (C) 2020 Red Hat (www.redhat.com)
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -39,11 +38,16 @@ struct _EBookBackendM365Private {
 	GRecMutex property_lock;
 	EM365Connection *cnc;
 	gchar *folder_id;
+	GHashTable *view_cancellables; /* gpointer view ~> GCancellable; shares property_lock */
 	EM365FolderKind folder_kind;
+	gboolean cached_for_offline;
+	guint max_people;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (EBookBackendM365, e_book_backend_m365, E_TYPE_BOOK_META_BACKEND)
 
+static void ebb_m365_check_source_properties (EBookBackendM365 *bbm365);
+
 static void
 ebb_m365_contact_get_string_attribute (EM365Contact *m365_contact,
 				       EContact *inout_contact,
@@ -264,6 +268,46 @@ ebb_m365_contact_get_address (EBookBackendM365 *bbm365,
 				}
 			}
 		}
+	} else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE) {
+		JsonArray *values = e_m365_contact_person_get_postal_addresses (m365_contact);
+
+		if (values) {
+			guint ii, len;
+
+			len = json_array_get_length (values);
+
+			for (ii = 0; ii < len; ii++) {
+				EM365Location *location = json_array_get_object_element (values, ii);
+				EM365PhysicalAddress *phys_address;
+
+				if (!location)
+					continue;
+
+				phys_address = e_m365_location_get_address (location);
+				if (!phys_address)
+					continue;
+
+				if (field_id == E_CONTACT_ADDRESS_WORK) {
+					if (e_m365_location_get_type (location) != E_M365_LOCATION_BUSINESS_ADDRESS)
+						continue;
+				} else if (field_id == E_CONTACT_ADDRESS_HOME) {
+					if (e_m365_location_get_type (location) != E_M365_LOCATION_HOME_ADDRESS)
+						continue;
+				} else if (field_id == E_CONTACT_ADDRESS_OTHER) {
+					if (e_m365_location_get_type (location) != E_M365_LOCATION_STREET_ADDRESS &&
+					    e_m365_location_get_type (location) != E_M365_LOCATION_POSTAL_ADDRESS)
+						continue;
+				}
+
+				addr.locality = (gchar *) e_m365_physical_address_get_city (phys_address);
+				addr.country = (gchar *) e_m365_physical_address_get_country_or_region (phys_address);
+				addr.code = (gchar *) e_m365_physical_address_get_postal_code (phys_address);
+				addr.region = (gchar *) e_m365_physical_address_get_state (phys_address);
+				addr.street = (gchar *) e_m365_physical_address_get_street (phys_address);
+				/* only one supported */
+				break;
+			}
+		}
 	}
 
 	if (addr.locality || addr.country || addr.code || addr.region || addr.street)
@@ -525,10 +569,15 @@ ebb_m365_contact_get_phone (EBookBackendM365 *bbm365,
 	} else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_USERS && field_id == E_CONTACT_PHONE_HOME) {
 		e_contact_set (inout_contact, E_CONTACT_PHONE_HOME_FAX, e_m365_contact_user_get_fax_number (m365_contact));
 		e_contact_set (inout_contact, E_CONTACT_PHONE_MOBILE, e_m365_contact_user_get_mobile_phone (m365_contact));
-	} else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_ORG_CONTACTS && field_id == E_CONTACT_PHONE_BUSINESS) {
+	} else if ((bbm365->priv->folder_kind == E_M365_FOLDER_KIND_ORG_CONTACTS || bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE) &&
+		   field_id == E_CONTACT_PHONE_BUSINESS) {
 		JsonArray *values;
 
-		values = e_m365_contact_org_get_phones (m365_contact);
+		if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_ORG_CONTACTS)
+			values = e_m365_contact_org_get_phones (m365_contact);
+		else
+			values = e_m365_contact_person_get_phones (m365_contact);
+
 		if (values) {
 			gboolean had_business = FALSE;
 			gboolean had_home = FALSE;
@@ -818,22 +867,8 @@ ebb_m365_contact_get_emails (EBookBackendM365 *bbm365,
 					attr = e_vcard_attribute_new (NULL, EVC_EMAIL);
 					e_vcard_attribute_add_param_with_value (attr, e_vcard_attribute_param_new (EVC_TYPE), "OTHER");
 
-					if (g_strcmp0 (e_m365_email_address_get_name (address), e_m365_email_address_get_address (address)) == 0) {
-						e_vcard_add_attribute_with_value (vcard, attr, e_m365_email_address_get_address (address));
-					} else {
-						gchar *formatted;
-
-						formatted = camel_internet_address_format_address (
-							e_m365_email_address_get_name (address),
-							e_m365_email_address_get_address (address));
-
-						if (formatted && *formatted)
-							e_vcard_add_attribute_with_value (vcard, attr, formatted);
-						else
-							e_vcard_attribute_free (attr);
-
-						g_free (formatted);
-					}
+					/* ignore the address name, it's not meant to be there, but the server sometimes sets it and sends it back */
+					e_vcard_add_attribute_with_value (vcard, attr, e_m365_email_address_get_address (address));
 				}
 			}
 		}
@@ -842,6 +877,7 @@ ebb_m365_contact_get_emails (EBookBackendM365 *bbm365,
 		EVCard *vcard = E_VCARD (inout_contact);
 		EVCardAttribute *attr;
 		JsonArray *proxy_addresses = NULL;
+		JsonArray *scored_addresses = NULL;
 		JsonArray *other_mails = NULL;
 		const gchar *mail = NULL;
 		guint ii, len;
@@ -853,22 +889,35 @@ ebb_m365_contact_get_emails (EBookBackendM365 *bbm365,
 			proxy_addresses = e_m365_contact_user_get_proxy_addresses (m365_contact);
 			other_mails = e_m365_contact_user_get_other_mails (m365_contact);
 			mail = e_m365_contact_user_get_mail (m365_contact);
+		} else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE) {
+			scored_addresses = e_m365_contact_person_get_scored_email_addresses (m365_contact);
 		}
 
 		/* add it to the vCard as the last, because "add_attribute" prepends it */
 		if (mail && *mail)
 			g_hash_table_add (known_mails, (gpointer) mail);
 
-		len = proxy_addresses ? json_array_get_length (proxy_addresses) : 0;
+		len = proxy_addresses ? json_array_get_length (proxy_addresses) :
+		      (scored_addresses ? json_array_get_length (scored_addresses) : 0);
 
 		for (ii = 0; ii < len; ii++) {
-			const gchar *address = json_array_get_string_element (proxy_addresses, len - ii - 1);
+			const gchar *address = NULL;
+
+			if (proxy_addresses) {
+				address = json_array_get_string_element (proxy_addresses, len - ii - 1);
+			} else if (scored_addresses) {
+				EM365ScoredEmailAddress *scored_address;
+
+				scored_address = json_array_get_object_element (scored_addresses, len - ii - 1);
+				if (scored_address)
+					address = e_m365_scored_email_address_get_address (scored_address);
+			}
 
 			if (address && *address) {
 				/* accept only SMTP: addresses and skip any other */
 				if (g_ascii_strncasecmp (address, "smtp:", 5) == 0)
 					address += 5;
-				else
+				else if (!scored_addresses)
 					continue;
 
 				if (g_hash_table_add (known_mails, (gpointer) address)) {
@@ -906,7 +955,6 @@ ebb_m365_contact_get_emails (EBookBackendM365 *bbm365,
 
 static gboolean
 ebb_m365_parse_qp_email (const gchar *string,
-			 gchar **name,
 			 gchar **email)
 {
 	struct _camel_header_address *address;
@@ -917,7 +965,6 @@ ebb_m365_parse_qp_email (const gchar *string,
 	if (address) {
 		/* report success only when we have filled both name and email address */
 		if (address->type == CAMEL_HEADER_ADDRESS_NAME && address->name && *address->name && address->v.addr && *address->v.addr) {
-			*name = g_strdup (address->name);
 			*email = g_strdup (address->v.addr);
 			res = TRUE;
 		}
@@ -932,7 +979,6 @@ ebb_m365_parse_qp_email (const gchar *string,
 		if (camel_address_unformat (CAMEL_ADDRESS (addr), string) == 1 &&
 		    camel_internet_address_get (addr, 0, &const_name, &const_email) &&
 		    const_name && *const_name && const_email && *const_email) {
-			*name = g_strdup (const_name);
 			*email = g_strdup (const_email);
 			res = TRUE;
 		}
@@ -965,14 +1011,13 @@ ebb_m365_contact_add_emails (EBookBackendM365 *bbm365,
 
 		for (link = new_values; link; link = g_list_next (link)) {
 			const gchar *value = link->data;
-			gchar *name = NULL, *address = NULL;
+			gchar *address = NULL;
 
-			if (ebb_m365_parse_qp_email (value, &name, &address))
-				e_m365_add_email_address (builder, NULL, name, address);
+			if (ebb_m365_parse_qp_email (value, &address))
+				e_m365_add_email_address (builder, NULL, NULL, address);
 			else
 				e_m365_add_email_address (builder, NULL, NULL, value);
 
-			g_free (name);
 			g_free (address);
 		}
 
@@ -1550,6 +1595,229 @@ ebb_m365_contact_to_json_2nd_go_locked (EBookBackendM365 *bbm365,
 	return success;
 }
 
+typedef struct {
+	gboolean is_autocompletion;
+	gchar *auto_comp_str;
+} EM365SExpData;
+
+static ESExpResult *
+ebb_m365_func_not (ESExp *f,
+		   gint argc,
+		   ESExpResult **argv,
+		   gpointer data)
+{
+	ESExpResult *r;
+
+	if (argc != 1 || argv[0]->type != ESEXP_RES_UNDEFINED) {
+		e_sexp_fatal_error (f, "parse error");
+		return NULL;
+	}
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+}
+
+static ESExpResult *
+ebb_m365_func_and_or (ESExp *f,
+		      gint argc,
+		      ESExpResult **argv,
+		      gpointer data)
+{
+	ESExpResult *r;
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+}
+
+static ESExpResult *
+ebb_m365_func_is (struct _ESExp *f,
+		  gint argc,
+		  struct _ESExpResult **argv,
+		  gpointer data)
+{
+	ESExpResult *r;
+
+	if (argc != 2
+	    && argv[0]->type != ESEXP_RES_STRING
+	    && argv[1]->type != ESEXP_RES_STRING) {
+		e_sexp_fatal_error (f, "parse error");
+		return NULL;
+	}
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+}
+
+static ESExpResult *
+ebb_m365_func_endswith (struct _ESExp *f,
+			gint argc,
+			struct _ESExpResult **argv,
+			gpointer data)
+{
+	ESExpResult *r;
+
+	if (argc != 2
+	    && argv[0]->type != ESEXP_RES_STRING
+	    && argv[1]->type != ESEXP_RES_STRING) {
+		e_sexp_fatal_error (f, "parse error");
+		return NULL;
+	}
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+
+}
+
+static ESExpResult *
+ebb_m365_func_contains (struct _ESExp *f,
+			gint argc,
+			struct _ESExpResult **argv,
+			gpointer data)
+{
+	ESExpResult *r;
+	EM365SExpData *sdata = data;
+	const gchar *propname, *str;
+
+	if (argc != 2
+	    && argv[0]->type != ESEXP_RES_STRING
+	    && argv[1]->type != ESEXP_RES_STRING) {
+		e_sexp_fatal_error (f, "parse error");
+		return NULL;
+	}
+
+	propname = argv[0]->value.string;
+	str = argv[1]->value.string;
+
+	if (!g_ascii_strcasecmp (propname, "full_name") || !g_ascii_strcasecmp (propname, "email") ||
+	    (str && *str && !g_ascii_strcasecmp (propname, "x-evolution-any-field"))) {
+		if (!sdata->auto_comp_str) {
+			sdata->auto_comp_str = g_strdup (str);
+			sdata->is_autocompletion = TRUE;
+		}
+	}
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+
+}
+
+/* We are just handling for autocompletion now. */
+static ESExpResult *
+ebb_m365_func_beginswith (struct _ESExp *f,
+			  gint argc,
+			  struct _ESExpResult **argv,
+			  gpointer data)
+{
+	ESExpResult *r;
+	const gchar *propname, *str;
+	EM365SExpData *sdata = data;
+
+	if (argc != 2 ||
+	    argv[0]->type != ESEXP_RES_STRING ||
+	    argv[1]->type != ESEXP_RES_STRING) {
+		e_sexp_fatal_error (f, "parse error");
+		return NULL;
+	}
+
+	propname = argv[0]->value.string;
+	str = argv[1]->value.string;
+
+	if (!g_ascii_strcasecmp (propname, "full_name") || !g_ascii_strcasecmp (propname, "email") ||
+	    (str && *str && !g_ascii_strcasecmp (propname, "x-evolution-any-field"))) {
+		if (!sdata->auto_comp_str) {
+			sdata->auto_comp_str = g_strdup (str);
+			sdata->is_autocompletion = TRUE;
+		}
+	}
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+	return r;
+}
+
+static ESExpResult *
+ebb_m365_func_exists (struct _ESExp *f,
+		      gint argc,
+		      struct _ESExpResult **argv,
+		      gpointer data)
+{
+	ESExpResult *r;
+
+	r = e_sexp_result_new (f, ESEXP_RES_BOOL);
+	r->value.boolean = FALSE;
+
+	return r;
+}
+
+static struct {
+	const gchar *name;
+	ESExpFunc *func;
+	guint flags;
+} symbols[] = {
+	{ "and", ebb_m365_func_and_or, 0 },
+	{ "or", ebb_m365_func_and_or, 0},
+	{ "not", ebb_m365_func_not, 0 },
+	{ "contains", ebb_m365_func_contains, 0},
+	{ "is", ebb_m365_func_is, 0},
+	{ "beginswith", ebb_m365_func_beginswith, 0},
+	{ "endswith", ebb_m365_func_endswith, 0},
+	{ "exists", ebb_m365_func_exists, 0}
+};
+
+static gchar *
+ebb_m365_expr_to_search_text (const gchar *query)
+{
+	ESExp *sexp;
+	gchar *text = NULL;
+	EM365SExpData sdata;
+	guint ii;
+
+	sexp = e_sexp_new ();
+	memset (&sdata, 0, sizeof (EM365SExpData));
+	sdata.is_autocompletion = FALSE;
+
+	for (ii = 0; ii < G_N_ELEMENTS (symbols); ii++) {
+		e_sexp_add_function (
+			sexp, 0, (gchar *) symbols[ii].name,
+			symbols[ii].func,
+			&sdata);
+	}
+
+	e_sexp_input_text (sexp, query, strlen (query));
+
+	if (e_sexp_parse (sexp) == -1) {
+		const gchar *errstr = e_sexp_get_error (sexp);
+
+		g_printerr ("%s: Failed to parse query '%s': %s\n", G_STRFUNC, query, errstr ? errstr : "Unknown error");
+	} else {
+		ESExpResult *r;
+
+		r = e_sexp_eval (sexp);
+		if (r) {
+			if (sdata.is_autocompletion && sdata.auto_comp_str && *sdata.auto_comp_str)
+				text = sdata.auto_comp_str;
+			else
+				g_free (sdata.auto_comp_str);
+		}
+
+		e_sexp_result_free (sexp, r);
+	}
+
+	g_object_unref (sexp);
+
+	return text;
+}
+
 static void
 ebb_m365_convert_error_to_client_error (GError **perror)
 {
@@ -1588,6 +1856,100 @@ ebb_m365_convert_error_to_client_error (GError **perror)
 	}
 }
 
+static gboolean
+ebb_m365_enum_cached_for_offline (EBookBackendM365 *bbm365)
+{
+	if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_ORG_CONTACTS ||
+	    bbm365->priv->folder_kind == E_M365_FOLDER_KIND_USERS) {
+		ESourceOffline *offline_ext;
+
+		offline_ext = e_source_get_extension (e_backend_get_source (E_BACKEND (bbm365)), E_SOURCE_EXTENSION_OFFLINE);
+		return e_source_offline_get_stay_synchronized (offline_ext);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+ebb_m365_update_cache_for_expression_sync (EBookBackendM365 *bbm365,
+					   const gchar *expr,
+					   GCancellable *cancellable,
+					   GError **error)
+{
+	EBookCache *book_cache;
+	GSList *contacts = NULL; /* EM365Contact * */
+	gchar *text;
+	gboolean success = TRUE;
+
+	if (bbm365->priv->cached_for_offline ||
+	    bbm365->priv->folder_kind == E_M365_FOLDER_KIND_CONTACTS ||
+	    bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE)
+		return success;
+
+	if (!expr || !*expr || g_ascii_strcasecmp (expr, "(contains \"x-evolution-any-field\" \"\")") == 0)
+		return success;
+
+	text = ebb_m365_expr_to_search_text (expr);
+	if (!text || !*text) {
+		g_free (text);
+		return success;
+	}
+
+	book_cache = e_book_meta_backend_ref_cache (E_BOOK_META_BACKEND (bbm365));
+	if (!E_IS_BOOK_CACHE (book_cache)) {
+		g_warn_if_fail (E_IS_BOOK_CACHE (book_cache));
+		g_free (text);
+		return FALSE;
+	}
+
+	LOCK (bbm365);
+
+	success = e_book_meta_backend_ensure_connected_sync (E_BOOK_META_BACKEND (bbm365), cancellable, error) &&
+		e_m365_connection_search_contacts_sync (bbm365->priv->cnc, NULL, bbm365->priv->folder_kind, bbm365->priv->folder_id, text, &contacts, cancellable, error);
+
+	if (success && contacts) {
+		EBookBackend *book_backend = E_BOOK_BACKEND (bbm365);
+		GSList *link;
+
+		for (link = contacts; link && success; link = g_slist_next (link)) {
+			EM365Contact *contact = link->data;
+			EContact *vcard;
+			gchar *object;
+			const gchar *id;
+
+			if (!contact)
+				continue;
+
+			id = e_m365_contact_get_id (contact);
+			if (!id)
+				continue;
+
+			vcard = ebb_m365_json_contact_to_vcard (bbm365, contact, bbm365->priv->cnc, &object, cancellable, error);
+
+			if (!g_cancellable_is_cancelled (cancellable))
+				g_warn_if_fail (object != NULL);
+
+			if (object && vcard) {
+				success = e_book_cache_put_contact (book_cache, vcard, object, 0, E_CACHE_IS_ONLINE, cancellable, error);
+
+				if (success)
+					e_book_backend_notify_update (book_backend, vcard);
+			}
+
+			g_clear_object (&vcard);
+			g_free (object);
+		}
+	}
+
+	UNLOCK (bbm365);
+
+	g_slist_free_full (contacts, (GDestroyNotify) json_object_unref);
+	g_clear_object (&book_cache);
+	g_free (text);
+
+	return success;
+}
+
 static void
 ebb_m365_maybe_disconnect_sync (EBookBackendM365 *bbm365,
 			        GError **in_perror,
@@ -1686,6 +2048,14 @@ ebb_m365_connect_sync (EBookMetaBackend *meta_backend,
 
 				bbm365->priv->folder_kind = E_M365_FOLDER_KIND_USERS;
 
+				/* do not store artificial folder id */
+				g_clear_pointer (&folder_id, g_free);
+			} else if (g_ascii_strcasecmp (folder_id, E_M365_ARTIFICIAL_FOLDER_ID_PEOPLE) == 0) {
+				*out_auth_result = e_m365_connection_authenticate_sync (cnc, NULL, E_M365_FOLDER_KIND_PEOPLE, NULL, NULL,
+					out_certificate_pem, out_certificate_errors, cancellable, error);
+
+				bbm365->priv->folder_kind = E_M365_FOLDER_KIND_PEOPLE;
+
 				/* do not store artificial folder id */
 				g_clear_pointer (&folder_id, g_free);
 			} else {
@@ -1705,6 +2075,8 @@ ebb_m365_connect_sync (EBookMetaBackend *meta_backend,
 
 				folder_id = NULL;
 				success = TRUE;
+
+				ebb_m365_check_source_properties (bbm365);
 			}
 		} else {
 			*out_auth_result = E_SOURCE_AUTHENTICATION_ERROR;
@@ -1821,10 +2193,14 @@ ebb_m365_get_changes_sync (EBookMetaBackend *meta_backend,
 
 	LOCK (bbm365);
 
-	success = e_m365_connection_get_objects_delta_sync (bbm365->priv->cnc, NULL,
-		bbm365->priv->folder_kind, bbm365->priv->folder_id, "id", last_sync_tag, 0,
-		ebb_m365_get_objects_delta_cb, &odd,
-		out_new_sync_tag, cancellable, &local_error);
+	if (bbm365->priv->cached_for_offline && bbm365->priv->folder_kind != E_M365_FOLDER_KIND_PEOPLE) {
+		success = e_m365_connection_get_objects_delta_sync (bbm365->priv->cnc, NULL,
+			bbm365->priv->folder_kind, bbm365->priv->folder_id, "id", last_sync_tag, 0,
+			ebb_m365_get_objects_delta_cb, &odd,
+			out_new_sync_tag, cancellable, &local_error);
+	} else {
+		success = TRUE;
+	}
 
 	if (e_m365_connection_util_delta_token_failed (local_error)) {
 		GSList *known_uids = NULL, *link;
@@ -1854,7 +2230,7 @@ ebb_m365_get_changes_sync (EBookMetaBackend *meta_backend,
 		g_propagate_error (error, local_error);
 	}
 
-	if (success && odd.ids->len) {
+	if (success && (odd.ids->len || bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE)) {
 		GPtrArray *contacts = NULL;
 
 		switch (bbm365->priv->folder_kind) {
@@ -1870,6 +2246,10 @@ ebb_m365_get_changes_sync (EBookMetaBackend *meta_backend,
 			success = e_m365_connection_get_users_sync (bbm365->priv->cnc, NULL,
 				odd.ids, &contacts, cancellable, error);
 			break;
+		case E_M365_FOLDER_KIND_PEOPLE:
+			success = e_m365_connection_get_people_sync (bbm365->priv->cnc, NULL,
+				bbm365->priv->max_people, &contacts, cancellable, error);
+			break;
 		default:
 			break;
 		}
@@ -2016,6 +2396,8 @@ ebb_m365_save_contact_sync (EBookMetaBackend *meta_backend,
 			text = _("Cannot modify organizational contact");
 		else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_USERS)
 			text = _("Cannot modify user contact");
+		else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE)
+			text = _("Cannot modify recent contact");
 
 		g_propagate_error (error, EC_ERROR_EX (E_CLIENT_ERROR_PERMISSION_DENIED, text));
 
@@ -2129,6 +2511,8 @@ ebb_m365_remove_contact_sync (EBookMetaBackend *meta_backend,
 			text = _("Cannot remove organizational contact");
 		else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_USERS)
 			text = _("Cannot remove user contact");
+		else if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE)
+			text = _("Cannot remove recent contact");
 
 		g_propagate_error (error, EC_ERROR_EX (E_CLIENT_ERROR_PERMISSION_DENIED, text));
 		success = FALSE;
@@ -2153,7 +2537,7 @@ ebb_m365_search_sync (EBookMetaBackend *meta_backend,
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_M365 (meta_backend), FALSE);
 
 	/* Ignore errors, just try its best */
-	/*ebb_m365_update_cache_for_expression (E_BOOK_BACKEND_M365 (meta_backend), expr, cancellable, NULL);*/
+	ebb_m365_update_cache_for_expression_sync (E_BOOK_BACKEND_M365 (meta_backend), expr, cancellable, NULL);
 
 	/* Chain up to parent's method */
 	return E_BOOK_META_BACKEND_CLASS (e_book_backend_m365_parent_class)->search_sync (meta_backend, expr, meta_contact, out_contacts, cancellable, error);
@@ -2169,7 +2553,7 @@ ebb_m365_search_uids_sync (EBookMetaBackend *meta_backend,
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_M365 (meta_backend), FALSE);
 
 	/* Ignore errors, just try its best */
-	/*ebb_m365_update_cache_for_expression (E_BOOK_BACKEND_M365 (meta_backend), expr, cancellable, NULL);*/
+	ebb_m365_update_cache_for_expression_sync (E_BOOK_BACKEND_M365 (meta_backend), expr, cancellable, NULL);
 
 	/* Chain up to parent's method */
 	return E_BOOK_META_BACKEND_CLASS (e_book_backend_m365_parent_class)->search_uids_sync (meta_backend, expr,
@@ -2184,10 +2568,12 @@ ebb_m365_get_backend_property (EBookBackend *book_backend,
 	g_return_val_if_fail (prop_name != NULL, NULL);
 
 	if (g_str_equal (prop_name, CLIENT_BACKEND_PROPERTY_CAPABILITIES)) {
+		gboolean cached_for_offline = ebb_m365_enum_cached_for_offline (E_BOOK_BACKEND_M365 (book_backend));
+
 		return g_strjoin (",",
 			"net",
 			"contact-lists",
-			"do-initial-query",
+			cached_for_offline ? "do-initial-query" : "",
 			e_book_meta_backend_get_capabilities (E_BOOK_META_BACKEND (book_backend)),
 			NULL);
 	} else if (g_str_equal (prop_name, E_BOOK_BACKEND_PROPERTY_REQUIRED_FIELDS)) {
@@ -2225,6 +2611,93 @@ ebb_m365_get_backend_property (EBookBackend *book_backend,
 	return E_BOOK_BACKEND_CLASS (e_book_backend_m365_parent_class)->impl_get_backend_property (book_backend, prop_name);
 }
 
+static void
+ebb_m365_start_view_search_thread_func (EBookBackend *book_backend,
+					gpointer user_data,
+					GCancellable *cancellable,
+					GError **error)
+{
+	EDataBookView *view = user_data;
+	EBookBackendM365 *bbm365;
+
+	g_return_if_fail (E_IS_BOOK_BACKEND_M365 (book_backend));
+	g_return_if_fail (E_IS_DATA_BOOK_VIEW (view));
+
+	bbm365 = E_BOOK_BACKEND_M365 (book_backend);
+
+	if (!g_cancellable_set_error_if_cancelled (cancellable, error)) {
+		EBookBackendSExp *sexp;
+		GSList *contacts = NULL;
+		const gchar *expr = NULL;
+
+		sexp = e_data_book_view_get_sexp (view);
+		if (sexp)
+			expr = e_book_backend_sexp_text (sexp);
+
+		/* this is called only for not-cached-for-offline books and it's enough to search for the contacts,
+		   the rest is done by the notifications when added/discovered a new contact */
+		if (e_book_meta_backend_search_sync (E_BOOK_META_BACKEND (book_backend), expr, TRUE, &contacts, cancellable, NULL))
+			g_slist_free_full (contacts, g_object_unref);
+	}
+
+	LOCK (bbm365);
+	g_hash_table_remove (bbm365->priv->view_cancellables, view);
+	UNLOCK (bbm365);
+}
+
+static void
+ebb_m365_start_view (EBookBackend *book_backend,
+		     EDataBookView *view)
+{
+	if ((e_data_book_view_get_flags (view) & E_BOOK_CLIENT_VIEW_FLAGS_MANUAL_QUERY) != 0) {
+		EBookBackendM365 *bbm365 = E_BOOK_BACKEND_M365 (book_backend);
+
+		if (!bbm365->priv->cached_for_offline) {
+			GCancellable *cancellable;
+
+			cancellable = g_cancellable_new ();
+
+			LOCK (bbm365);
+			g_hash_table_insert (bbm365->priv->view_cancellables, view, g_object_ref (cancellable));
+			UNLOCK (bbm365);
+
+			e_book_backend_schedule_custom_operation (book_backend, cancellable,
+				ebb_m365_start_view_search_thread_func, g_object_ref (view), g_object_unref);
+
+			g_clear_object (&cancellable);
+		}
+	}
+
+	/* Chain up to parent's method. */
+	E_BOOK_BACKEND_CLASS (e_book_backend_m365_parent_class)->impl_start_view (book_backend, view);
+}
+
+static void
+ebb_m365_stop_view (EBookBackend *book_backend,
+		    EDataBookView *view)
+{
+	if ((e_data_book_view_get_flags (view) & E_BOOK_CLIENT_VIEW_FLAGS_MANUAL_QUERY) != 0) {
+		EBookBackendM365 *bbm365 = E_BOOK_BACKEND_M365 (book_backend);
+		GCancellable *cancellable;
+		gpointer value = NULL;
+
+		LOCK (bbm365);
+		if (!g_hash_table_steal_extended (bbm365->priv->view_cancellables, view, NULL, &value))
+			value = NULL;
+		UNLOCK (bbm365);
+
+		cancellable = value;
+
+		if (cancellable) {
+			g_cancellable_cancel (cancellable);
+			g_clear_object (&cancellable);
+		}
+	}
+
+	/* Chain up to parent's method. */
+	E_BOOK_BACKEND_CLASS (e_book_backend_m365_parent_class)->impl_stop_view (book_backend, view);
+}
+
 static gboolean
 ebb_m365_get_destination_address (EBackend *backend,
 				  gchar **host,
@@ -2244,6 +2717,43 @@ ebb_m365_get_destination_address (EBackend *backend,
 	return TRUE;
 }
 
+static void
+ebb_m365_check_source_properties (EBookBackendM365 *bbm365)
+{
+	ESourceM365Folder *m365_folder_ext;
+	guint max_people;
+	gboolean cached_for_offline;
+
+	if (!e_backend_get_source (E_BACKEND (bbm365)))
+		return;
+
+	cached_for_offline = ebb_m365_enum_cached_for_offline (bbm365);
+
+	if ((cached_for_offline ? 1 : 0) != (bbm365->priv->cached_for_offline ? 1 : 0)) {
+		EBookBackend *book_backend = E_BOOK_BACKEND (bbm365);
+		gchar *value;
+
+		bbm365->priv->cached_for_offline = cached_for_offline;
+
+		value = ebb_m365_get_backend_property (book_backend, CLIENT_BACKEND_PROPERTY_CAPABILITIES);
+
+		e_book_backend_notify_property_changed (book_backend, CLIENT_BACKEND_PROPERTY_CAPABILITIES, value);
+
+		g_free (value);
+	}
+
+	m365_folder_ext = e_source_get_extension (e_backend_get_source (E_BACKEND (bbm365)), E_SOURCE_EXTENSION_M365_FOLDER);
+	max_people = e_source_m365_folder_get_max_people (m365_folder_ext);
+
+	if (max_people != bbm365->priv->max_people) {
+		bbm365->priv->max_people = max_people;
+
+		if (bbm365->priv->folder_kind == E_M365_FOLDER_KIND_PEOPLE &&
+		    e_backend_get_online (E_BACKEND (bbm365)))
+			e_book_meta_backend_schedule_refresh (E_BOOK_META_BACKEND (bbm365));
+	}
+}
+
 static void
 e_book_backend_m365_dispose (GObject *object)
 {
@@ -2261,6 +2771,7 @@ e_book_backend_m365_finalize (GObject *object)
 	EBookBackendM365 *bbm365 = E_BOOK_BACKEND_M365 (object);
 
 	g_rec_mutex_clear (&bbm365->priv->property_lock);
+	g_hash_table_destroy (bbm365->priv->view_cancellables);
 
 	/* Chain up to parent's method. */
 	G_OBJECT_CLASS (e_book_backend_m365_parent_class)->finalize (object);
@@ -2272,6 +2783,10 @@ e_book_backend_m365_init (EBookBackendM365 *bbm365)
 	bbm365->priv = e_book_backend_m365_get_instance_private (bbm365);
 
 	g_rec_mutex_init (&bbm365->priv->property_lock);
+	bbm365->priv->view_cancellables = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
+
+	g_signal_connect (bbm365, "source-changed",
+		G_CALLBACK (ebb_m365_check_source_properties), NULL);
 }
 
 static void
@@ -2296,6 +2811,8 @@ e_book_backend_m365_class_init (EBookBackendM365Class *klass)
 
 	book_backend_class = E_BOOK_BACKEND_CLASS (klass);
 	book_backend_class->impl_get_backend_property = ebb_m365_get_backend_property;
+	book_backend_class->impl_start_view = ebb_m365_start_view;
+	book_backend_class->impl_stop_view = ebb_m365_stop_view;
 
 	backend_class = E_BACKEND_CLASS (klass);
 	backend_class->get_destination_address = ebb_m365_get_destination_address;
