@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * SPDX-FileCopyrightText: (C) 2020 Red Hat (www.redhat.com)
  * SPDX-License-Identifier: LGPL-2.1-or-later
@@ -181,6 +180,15 @@ m365_backend_update_resource (EM365Backend *m365_backend,
 				g_free (today);
 			}
 
+			if (g_strcmp0 (id, E_M365_ARTIFICIAL_FOLDER_ID_ORG_CONTACTS) == 0 ||
+			    g_strcmp0 (id, E_M365_ARTIFICIAL_FOLDER_ID_USERS) == 0) {
+				ESourceOffline *offline_ext;
+
+				offline_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
+				/* look up by default, these can be very large */
+				e_source_offline_set_stay_synchronized (offline_ext, FALSE);
+			}
+
 			e_source_m365_folder_set_id (folder_ext, id);
 			e_source_m365_folder_set_group_id (folder_ext, group_id);
 			e_source_m365_folder_set_is_default (folder_ext, is_default);
@@ -190,6 +198,14 @@ m365_backend_update_resource (EM365Backend *m365_backend,
 			e_source_registry_server_add_source (server, source);
 
 			g_clear_object (&server);
+		} else if (!e_source_has_extension (source, E_SOURCE_EXTENSION_OFFLINE) && (
+			   g_strcmp0 (id, E_M365_ARTIFICIAL_FOLDER_ID_ORG_CONTACTS) == 0 ||
+			   g_strcmp0 (id, E_M365_ARTIFICIAL_FOLDER_ID_USERS) == 0)) {
+			ESourceOffline *offline_ext;
+
+			offline_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
+			/* existing books without the extension mean they had been synchronized for offline use */
+			e_source_offline_set_stay_synchronized (offline_ext, TRUE);
 		}
 	}
 
@@ -295,6 +311,7 @@ m365_backend_forget_book_folders (EM365Backend *m365_backend,
 
 	g_hash_table_remove (ids, E_M365_ARTIFICIAL_FOLDER_ID_ORG_CONTACTS);
 	g_hash_table_remove (ids, E_M365_ARTIFICIAL_FOLDER_ID_USERS);
+	g_hash_table_remove (ids, E_M365_ARTIFICIAL_FOLDER_ID_PEOPLE);
 
 	m365_backend_forget_folders_hash (m365_backend, extension_name, ids);
 
@@ -402,6 +419,13 @@ m365_backend_sync_contact_folders_sync (EM365Backend *m365_backend,
 	} else {
 		m365_backend_remove_resource (m365_backend, E_SOURCE_EXTENSION_ADDRESS_BOOK, E_M365_ARTIFICIAL_FOLDER_ID_USERS);
 	}
+
+	if (e_m365_connection_get_people_accessible_sync (cnc, NULL, cancellable, NULL)) {
+		m365_backend_update_resource (m365_backend, E_SOURCE_EXTENSION_ADDRESS_BOOK,
+			E_M365_ARTIFICIAL_FOLDER_ID_PEOPLE, NULL, _("Recent Contacts"), TRUE, NULL);
+	} else {
+		m365_backend_remove_resource (m365_backend, E_SOURCE_EXTENSION_ADDRESS_BOOK, E_M365_ARTIFICIAL_FOLDER_ID_PEOPLE);
+	}
 }
 
 static void
